#include <vector>
#include <stdexcept>
#include <iostream>

enum
{
    MAX_MAGASSAG = 32,
    MAX_SZELESSEG = 28,
};

char kezdo_palya[MAX_MAGASSAG][MAX_SZELESSEG + 1] =
{
    "############################",
    "#            ##            #",
    "# #### ##### ## ##### #### #",
    "# #### ##### ## ##### #### #",
    "# #### ##### ## ##### #### #",
    "#                          #",
    "# #### ## ######## ## #### #",
    "# #### ## ######## ## #### #",
    "#      ##    ##    ##      #",
    "###### ##### ## ##### ######",
    "###### ##### ## ##### ######",
    "###### ##          ## ######",
    "###### ## ##    ## ## ######",
    "###### ## #      # ## ######",
    "###### ## #      # ## ######",
    "          #      #          ",
    "###### ## #      # ## ######",
    "###### ## #      # ## ######",
    "###### ## ##    ## ## ######",
    "###### ##          ## ######",
    "###### ## ######## ## ######",
    "#            ##            #",
    "# #### ##### ## ##### #### #",
    "# #### ##### ## ##### #### #",
    "#   ##                ##   #",
    "### ## ## ######## ## ## ###",
    "### ## ## ######## ## ## ###",
    "#      ##    ##    ##      #",
    "# ########## ## ########## #",
    "# ########## ## ########## #",
    "#                          #",
    "############################",
};

enum Irany
{
    IRANY_BAL = -2,
    IRANY_FEL = -1,
    IRANY_LE = 1,
    IRANY_JOBB = 2,
    IRANY_SEMMI = 0,
};

Irany irany_random()
{
    return Irany(rand() % 5 - 2);
}

struct Pont
{
    int x;
    int y;
    Pont() : x(0), y(0) {}
    Pont(int x, int y) : x(x), y(y) {}
};

Pont operator+(Pont p, Irany merre) {
    return Pont(p.x + merre / 2, p.y + merre % 2); // kihasználva az enum->int leképezés értékeit
}

class Palya
{
    int m;
    int sz;
    std::vector<char> matrix;

public:
    Palya() : m(0), sz(0) {}

    Palya(int magassag, int szelesseg)
        : m(magassag)
        , sz(szelesseg)
        , matrix(m * sz, ' ')
    {
    }

    template<size_t M, size_t SZ>
    explicit Palya(const char (&palya)[M][SZ])
        : m(M)
        , sz(SZ-1) // lezáró '\0' miatt -1
        , matrix(m * sz)
    {
        for (int y = 0; y < m; ++y)
            for (int x = 0; x < sz; ++x)
                (*this)[Pont(x, y)] = palya[y][x];
    }

    char& operator[](Pont p)
    {
        return matrix[p.y * sz + p.x];
    }

    const char& operator[](Pont p) const
    {
        return matrix[p.y * sz + p.x];
    }

    int szelesseg() const
    {
        return sz;
    }
    int magassag() const
    {
        return m;
    }

    bool valid(Pont p) const
    {
        return p.x >= 0 && p.y >= 0 && p.x < sz && p.y < m;
    }

    bool szabad(Pont p) const
    {
        return valid(p) && (*this)[p] == ' ';
    }

    Pont kovetkezo(Pont honnan, Irany merre) const
    {
        if (!valid(honnan))
            throw std::invalid_argument("Tabla::kovetkezo");

        Pont uj = honnan + merre;
        if (uj.x < 0)   uj.x = sz - 1;
        if (uj.x >= sz) uj.x = 0;
        if (uj.y < 0)   uj.y = m - 1;
        if (uj.y >= m)  uj.y = 0;

        return uj;
    }
};

std::ostream& operator<<(std::ostream& os, Palya const& p)
{
    for (int y = 0; y < p.magassag(); ++y)
    {
        for (int x = 0; x < p.szelesseg(); ++x)
            os << p[Pont(x, y)]; // C++11: p[{i, j}]
        os << '\n';
    }
    return os;
}

int main()
{
    Palya p(kezdo_palya);
    std::cout << p << std::endl;
}
